#include <iostream>
#include "3dengfx/3dengfx.hpp"
#include "common/timer.h"

bool Init();
void UpdateGraphics();
void CleanUp();
void KeyHandler(int key);


Scene *scene;
ntimer timer;

int main(int argc, char **argv) {
	if(!Init()) {
		return -1;
	}
	return fxwt::MainLoop();
}


bool Init() {
	GraphicsInitParameters *gip;
	if(!(gip = LoadGraphicsContextConfig("3dengfx.conf"))) {
		return false;
	}

	if(!CreateGraphicsContext(*gip)) {
		return false;
	}

	fxwt::SetWindowTitle("The Lab Demos :: 3dengfx sample");

	fxwt::SetDisplayHandler(UpdateGraphics);
	fxwt::SetIdleHandler(UpdateGraphics);
	fxwt::SetKeyboardHandler(KeyHandler);
	atexit(CleanUp);

	SceneLoader::SetDataPath("data/");	// where to find the textures actually

	// load the scene
	if(!SceneLoader::LoadScene("data/test_scene.3ds", &scene)) {
		std::cerr << "could not load scene file \"test_scene.3ds\"\n";
		return false;
	}

	scene->GetObject("logo")->SetDynamic(true);	// VBO bug? doing this seems to make it go away...

	// create the motion controller for the camera
	MotionController cam_motion = MotionController(scene->GetCurve("Line02"), 0, 20800, TIME_BOUNCE);
	
	// get a pointer to the camera object that we wish to assign the controller to
	Camera *cam = (TargetCamera*)scene->GetCamera("Camera01");
	
	/* the final position is calculated as the sum of all controllers
	 * plus the static position, so we zero that out to make the 
	 * controller position absolute.
	 */
	cam->SetPosition(Vector3(0, 0, 0));
	cam->AddController(cam_motion, CTRL_TRANSLATION);	// and assign it...

	timer_reset(&timer);
	timer_start(&timer);
	return true;
}

void CleanUp() {
	delete scene;
	
	DestroyGraphicsContext();
}

void UpdateGraphics() {
	unsigned long msec = timer_getmsec(&timer);

	Clear(Color(0.0f, 0.0f, 0.0f));
	ClearZBuffer(1.0f);

	scene->Render(msec);

	Flip();
}

void KeyHandler(int key) {
	switch(key) {
	case '`':
		ScreenCapture();
		break;

	case 27:
		exit(0);

	default:
		break;
	}
}
